/*
 * NonEmptyList.h
 *
 *  Created on: Oct 14, 2008
 *      Author: bloch
 */

#ifndef NONEMPTYLIST_H_
#define NONEMPTYLIST_H_

#include "List.h"

template<typename ElementType> class NonEmptyList: public List<ElementType> {
private:
	ElementType first;
	List<ElementType> *rest;
public:
	NonEmptyList<ElementType>(ElementType firstArg, List<ElementType> *restArg) : first(firstArg), rest(restArg) {}
	ElementType getFirst() const {
		return this->first;
	}
	List<ElementType> *getRest() const;
	virtual int length() const;
	virtual bool isEmpty() const {
		return false;
	}
	virtual std::ostream& printList (std::ostream &out) const {
		if (this->rest->isEmpty()) {
			out << this->first;
			return out;
		}
		else {
			out << this->first << ", ";
			return this->rest->printList(out);
		}
	}

	virtual bool operator==(const List<ElementType> &other) const {
		if (other.isEmpty()) {
			return false;
		}
		else {
			const NonEmptyList<ElementType> &otherNEL = dynamic_cast<const NonEmptyList<ElementType> &>(other);
			// equivalent to
			// final NonEmptyList<ElementType> otherNEL = (NonEmptyList<ElementType>)other;
			// in Java

			return (this->first == otherNEL.getFirst()) &&
				   (*(this->rest) == *(otherNEL.getRest()));
			// note if you leave out the *'s, it does a pointer comparison
			// rather than calling this function recursively
		}
	}

	virtual ~NonEmptyList() {
		delete this->rest; // calls its destructor, which deletes its rest, which calls its destructor, which ...
	}
};


// I defined getFirst inside the class above.  Just to show the difference,
// I'll define getRest outside the class instead.
template<typename ElementType> List<ElementType> *NonEmptyList<ElementType>::getRest() const {
	return this->rest;
}

// And likewise for length.
template<typename ElementType> int NonEmptyList<ElementType>::length() const
{
	return 1 + this->rest->length();
}



#endif /* NONEMPTYLIST_H_ */
